
[![Build Status](https://secure.travis-ci.org/mrkamel/stalking.png?branch=master)](http://travis-ci.org/mrkamel/stalking)
[![Code Quality](https://codeclimate.com/github/mrkamel/stalking.png)](https://codeclimate.com/github/mrkamel/stalking)
[![Gem Version](https://badge.fury.io/rb/stalking.svg)](http://badge.fury.io/rb/stalking)
[![Still Maintained](http://stillmaintained.com/mrkamel/stalking.png)](http://stillmaintained.com/mrkamel/stalking)
[![Dependency Status](https://gemnasium.com/mrkamel/stalking.png?travis)](https://gemnasium.com/mrkamel/stalking)

# Stalking

Asynchronous job processing using the beanstalkd queue server with automatic
reconnects and failover for your ruby projects.

Background jobs are essential to every project these days and the tools should
be as reliable as possible.

Beanstalkd is perfectly suited for background processing and the stalker gem
has long been my favorite to integrate beanstalkd into my projects.
Unfortunately, stalker does not provide automatic reconnects and failover.

The DSL provided by stalking is very similar to the DSL of stalker with only
minimal changes driven by design decisions and its additional feature set.

## Installation

Add this line to your application's Gemfile:

    gem 'stalking'

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install stalking

## Usage

Stalking provides two modes of operation, a producer and a consumer mode.

Enqueuing new jobs is as easy as:

```ruby
Stalking::Producer.new.enqueue "name", "key" => "value"
```

This will try to connect to a beanstalkd instance running on `localhost:11300`
to enqueue a new job named `job` with the provided arguments. The arguments
will be serialized using JSON before being enqueued.

To specify a `priority`, `ttr` or `delay`, please use:

```ruby
Stalking::Producer.new.enqueue "name", { "key" => "value" }, :ttr => 360, :delay => 60, :pri => 100
```

You can of course use multiple beanstalkd instances, by using:

```ruby
producer = Stalking::Producer.new(:servers => ["192.168.1.100:11300", "192.168.1.101"])
producer.enqueue "name", "key" => "value"
```

Here, everytime you enqueue a new job, stalking will randomly choose a server
from your list to send the job to. Thus, you can e.g. create a producer for
local jobs (like e.g. for local background image processing) and another
producer for global jobs:

```ruby
global_queue = Stalking::Producer.new(:servers => ["192.168.1.100:11300", "192.168.1.101"])
local_queue = Stalking::Producer.new
```

If a server from your list is down or unreachable, stalking will use another
one from your list. Only in case all your servers are down, `enqueue` will
return false. However, please carefully read the `Caveats` section.

You can even set the `ttr`, `priority` and `delay` options
on a per-producer base:

```ruby
producer = Stalking::Producer.new(:pri => 100, :delay => 60, :ttr => 360)
```

To set an upper limit for servers to be tried out for queuing use the `tries`
option:

```ruby
producer = Stalking::Producer.new(:servers => [..., ..., ...], :tries => 2)
```

To consume jobs in a background worker, please use the following DSL, which is
quite similar to stalker's DSL:

```ruby
Stalking::Consumer.new do
  job "name" do |args|
    # Do something
  end

  job "another" do |args|
    # Do something
  end

  ...
end
```

After being specified, the consumer will immediately block and wait to consume
jobs from beanstalkd.

Additionally, you can specify before, after and error hooks:

```ruby
Stalking::Consumer.new do
  before do |name, args|
    # Executed before a job runs
  end

  after do |name, args|
    # Executed after a job has run
  end

  error do |e, name, args|
    # Executed when job exited abnormally
  end

  job "name" do |args|
    # The job itself
  end
end
```

If you save this file to e.g. `jobs.rb`, you can execute your worker like:

```
$ stalking jobs
```

Again, by default stalking connects to localhost:11300, but you can specify a
pool of servers to consume jobs from:

```ruby
Stalking::Consumer.new(:servers => ["192.168.1.100:11300", "192.168.1.101:11300"]) do
  ...
end
```

Please note that, due to the implementation of beanstalk-client, the consumer
requires all pool servers to be up. Otherwise, the consumer will enter a
reconnect loop, until all servers are up again. Thus, stalking will try to
reconnect to the servers when a connection is lost and it's recommended for
consumers to connect to a single server, usually.

That should be it.

## Caveats

Having producers which try to reconnect and/or try to connect to other servers
when a single server is down, of course require some time for the error
detection and processing. This usually should not be a big issue, except there
is some temporary lag between the producer and the beanstalkd server. The
producer then has to wait for a timeout and while the producer is waiting for
the timeout it will block your application. The default timeout is 1 second,
but you can change the timeout via:

```ruby
Stalking::Producer.new :timeout => 0.5 # Set timeout to 500 milliseconds
```

The optimal timeout for your setup depends on your network connection, the
distance of your servers, etc. You should experiment a bit with it if you
experience some kind of problems. However, i don't emphasize to use stalking
producers which communicate with beanstalkd servers over WAN connections.
Only local, data-center connections to beanstalkd servers should be used.

# Error Logging

To log connection errors, you can provide a logger to the consumers as well
as producers:

```ruby
Stalking::Producer.new :logger => ...
Stalking::Consumer.new :logger => ...
```

You can hand out any kind of logging object, which responds to an `error`
method and takes a string error message.

# Graceful termination

To stop a worker gracefully, send a QUIT or USR2 signal to it. The worker will
finish its current job and terminate afterwards.

## Contributing

1. Fork it
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create new Pull Request

